# -*- coding: utf-8 -*-
"""
Created on 18 Nov 2019

@author: etijskens

Module et_micc.utils
====================

Utility functions for et_micc.
"""
import os
import re
import subprocess
import copy
from pathlib import Path
from contextlib import contextmanager

from et_micc.tomlfile import TomlFile
import et_micc.logging


def replace_in_file(file_to_search, look_for, replace_with):
    path = Path(file_to_search)
    text = path.read_text()
    text = text.replace(look_for, replace_with)
    path.write_text(text)


def constraint_to_version(constraint):

    for i,c in enumerate(constraint):
        if c.isdigit():
            break
    tpl = constraint[i:].split('.')
    return tpl

    
def compare_constraints(left,right):
    """
    this fails for constraints with < and <=
    """
    left  = constraint_to_version(left)
    right = constraint_to_version(right)
    for i,j in zip(left,right):
        if i<j:
            return -1
        if i>j:
            return 1
    return 0

    
def verify_project_name(project_name):
    """Project names must start with a char, and  contain only chars, digits, underscores and dashes.
    
    :returns: bool
    """
    p = re.compile("\A[a-zA-Z][a-zA-Z0-9_-]*\Z")
    return bool( p.match(project_name) )
    
    
def pep8_module_name(name):
    """Convert a module name to a PEP8 compliant module name.
    
    * lowercase
    * whitespace -> underscore 
    * dash -> underscore 
    """
    if name[0].isnumeric():
        name = '_'+name
    
    valid_module_name = name.lower().replace('-', '_').replace(' ', '_')            
    
    return valid_module_name


def is_project_directory(path,project=None):
    """Verify that the directory :file:`path` is a project directory. 
    
    :param Path path: path to a directory.
    :returns: bool.
    
    As a sufficident condition, we request that 
    
    * there is a pyproject.toml file, exposing ``['tool']['poetry']['name']``
    * that there is a python package or module with that name.
    """
    if not isinstance(path, Path):
        path = Path(path)
        
    path_to_pyproject_toml = str(path /'pyproject.toml')
    
    try:
        pyproject_toml = TomlFile(path_to_pyproject_toml)
        project_name = pyproject_toml['tool']['poetry']['name']
        if not project is None:
            project.pyproject_toml = pyproject_toml
            project.project_name = project_name
    except Exception:
        return False

    return verify_project_structure(path,project)


def verify_project_structure(path,project=None):
    """
    :returns: a list, which should have length 1. If its length is 0, neither module.py, 
        nor module/__init__.py were found. If its length is 2, both were found.
    """
    package_name = pep8_module_name(path.name)
    
    module = path / (package_name + ".py")
    module = str(module.relative_to(path)) if module.exists() else ""

    package = path / package_name / "__init__.py"
    package = str(package.relative_to(path)) if package.exists() else ""
    
    if package and module:
        if project:
            project.error(f"Package ({package_name}/__init__.py) and module ({package_name}.py) found.")
        return False
    elif (not module and not package):
        if project:
            project.error(f"Neither package ({package_name}/__init__.py) nor module ({package_name}.py) found.")
        return False
    else:
        if project:
            project.module = module
            project.package = package
            project.package_name = package_name
        return True

@contextmanager
def in_directory(path):
    """Context manager for changing the current working directory while the body of the
    context manager executes.
    """
    previous_dir = os.getcwd()
    os.chdir(str(path)) # the str method takes care of when path is a Path object
    yield os.getcwd()
    os.chdir(previous_dir)


def execute(cmds,logfun=None,stop_on_error=True,env=None):
    """Executes a list of OS commands, and log with logfun.
    
    :param list cmds: list of OS commands (=list of list of str) or a single command (list of str)
    :parma callable logfun: a function to write output, typically 
        ``logging.getLogger('et_micc').debug``.
    :returns int: return code of first failing command, or 0 if all
        commanbds succeed.
    """
    if isinstance(cmds[0],str):
        # this is a single command
        cmds = [cmds]
        
    for cmd in cmds:
        with et_micc.logging.log(logfun, f"> {' '.join(cmd)}"):
            completed_process = subprocess.run(cmd, capture_output=True,env=env)
            if not logfun is None:
                if completed_process.returncode:
                    logfun0 = logfun
                    logfun = logfun.__self__.warning
                    logfun(f"> {' '.join(cmd)}")
                if completed_process.stdout:
                    logfun(' (stdout)\n' + completed_process.stdout.decode('utf-8'))
                if completed_process.stderr:
                    logfun(' (stderr)\n' + completed_process.stderr.decode('utf-8'))
                if completed_process.returncode:
                    logfun = logfun0
            if completed_process.returncode:
                if stop_on_error:
                    return completed_process.returncode
    return 0


def get_project_path(p):
    """Look for a project directory in the parents of path :py:obj:`p`.
    
    :param Path p:
    :returns: the nearest directory above :py:obj:`p` that is project directory.
    :raise: RuntimeError if :py:obj:`p` is noe inside a project directory.
    """
    root = Path('/')
    p = Path(p).resolve()
    p0 = copy.copy(p)
    while not is_project_directory(p):
        p = p.parent
        if p==root:
            raise RuntimeError(f"Folder {p0} is not inside a Python project.")
    return p


def insert_in_file(file, lines=[], before=False, startswith=None):
    """Insert *lines* at a specific position in a <file>.
    
    :param Path file: path to file in which to insert
    :param list lines: list of lines to insert. If a line does not end with 
        a newline, it is added.
    :param bool before: insert before or after a reference line.
    :param str startswith: find the reference line as the first line that
        starts with <startswith>.
    """
    if lines:
        with file.open() as f:
            content = f.readlines()
        for l,line in enumerate(content):
            if startswith and line.startswith(startswith):
                if not before:
                    l += 1
                break
        for i,line in enumerate(lines):
            if not line.endswith('\n'):
                line += '\n'
            content.insert(l+i,line)
        with file.open(mode='w') as f:
            for line in content:
                f.write(line)
        
    
#eof


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['et_micc']

package_data = \
{'': ['*'],
 'et_micc': ['templates/app-simple/{{cookiecutter.project_name}}/tests/test_{{cookiecutter.cli_app_name}}.{{cookiecutter.py}}',
             'templates/app-simple/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/*',
             'templates/app-sub-commands/{{cookiecutter.project_name}}/tests/test_{{cookiecutter.cli_app_name}}.{{cookiecutter.py}}',
             'templates/app-sub-commands/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/*',
             'templates/module-cpp/{{cookiecutter.project_name}}/tests/test_cpp_{{cookiecutter.module_name}}.{{cookiecutter.py}}',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/CMakeLists.txt',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/CMakeLists.txt',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/CMakeLists.txt',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.cpp',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.cpp',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.cpp',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.rst',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.rst',
             'templates/module-cpp/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/cpp_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.rst',
             'templates/module-f2py/{{cookiecutter.project_name}}/tests/test_f2py_{{cookiecutter.module_name}}.{{cookiecutter.py}}',
             'templates/module-f2py/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/f2py_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.f90',
             'templates/module-f2py/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/f2py_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.f90',
             'templates/module-f2py/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/f2py_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.rst',
             'templates/module-f2py/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/f2py_{{cookiecutter.module_name}}/{{cookiecutter.module_name}}.rst',
             'templates/module-py/{{cookiecutter.project_name}}/tests/test_{{cookiecutter.module_name}}.{{cookiecutter.py}}',
             'templates/module-py/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/{{cookiecutter.module_name}}.{{cookiecutter.py}}',
             'templates/package-base/hooks/*',
             'templates/package-base/{{cookiecutter.project_name}}/.gitignore',
             'templates/package-base/{{cookiecutter.project_name}}/.gitignore',
             'templates/package-base/{{cookiecutter.project_name}}/.gitignore',
             'templates/package-base/{{cookiecutter.project_name}}/.gitignore',
             'templates/package-base/{{cookiecutter.project_name}}/LICENSE',
             'templates/package-base/{{cookiecutter.project_name}}/LICENSE',
             'templates/package-base/{{cookiecutter.project_name}}/LICENSE',
             'templates/package-base/{{cookiecutter.project_name}}/LICENSE',
             'templates/package-base/{{cookiecutter.project_name}}/Makefile',
             'templates/package-base/{{cookiecutter.project_name}}/Makefile',
             'templates/package-base/{{cookiecutter.project_name}}/Makefile',
             'templates/package-base/{{cookiecutter.project_name}}/Makefile',
             'templates/package-base/{{cookiecutter.project_name}}/pyproject.toml',
             'templates/package-base/{{cookiecutter.project_name}}/pyproject.toml',
             'templates/package-base/{{cookiecutter.project_name}}/pyproject.toml',
             'templates/package-base/{{cookiecutter.project_name}}/pyproject.toml',
             'templates/package-base/{{cookiecutter.project_name}}/tests/__init__.{{cookiecutter.py}}',
             'templates/package-base/{{cookiecutter.project_name}}/tests/__init__.{{cookiecutter.py}}',
             'templates/package-base/{{cookiecutter.project_name}}/tests/test_{{cookiecutter.package_name}}.{{cookiecutter.py}}',
             'templates/package-base/{{cookiecutter.project_name}}/tests/test_{{cookiecutter.package_name}}.{{cookiecutter.py}}',
             'templates/package-general-docs/hooks/*',
             'templates/package-general-docs/{{cookiecutter.project_name}}/APPS.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/APPS.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/APPS.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/AUTHORS.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/AUTHORS.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/AUTHORS.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/HISTORY.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/HISTORY.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/HISTORY.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/apps.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/apps.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/apps.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/apps.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/apps.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/authors.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/authors.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/authors.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/authors.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/authors.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/history.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/history.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/history.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/history.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/history.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/index.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/index.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/index.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/index.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/index.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/installation.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/installation.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/installation.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/installation.rst',
             'templates/package-general-docs/{{cookiecutter.project_name}}/docs/installation.rst',
             'templates/package-general/hooks/*',
             'templates/package-general/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}/*',
             'templates/package-simple-docs/hooks/*',
             'templates/package-simple-docs/{{cookiecutter.project_name}}/API.rst',
             'templates/package-simple-docs/{{cookiecutter.project_name}}/API.rst',
             'templates/package-simple-docs/{{cookiecutter.project_name}}/README.rst',
             'templates/package-simple-docs/{{cookiecutter.project_name}}/README.rst',
             'templates/package-simple-docs/{{cookiecutter.project_name}}/docs/*',
             'templates/package-simple/hooks/*',
             'templates/package-simple/{{cookiecutter.project_name}}/{{cookiecutter.package_name}}.{{cookiecutter.py}}']}

install_requires = \
['click>=7.0,<8.0',
 'cookiecutter>=1.6.0,<2.0.0',
 'semantic_version>=2.8.3,<3.0.0',
 'sphinx-click>=2.3.0,<3.0.0',
 'sphinx-rtd-theme>=0.4.3,<0.5.0',
 'tomlkit>=0.5.8,<0.6.0',
 'walkdir>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['micc = et_micc:cli_micc.main']}

setup_kwargs = {
    'name': 'et-micc',
    'version': '0.9.35',
    'description': 'A practical Python project skeleton generator.',
    'long_description': '****\nMicc\n****\n\nThis is work in progress!\n\n.. image:: https://img.shields.io/pypi/v/micc.svg\n        :target: https://pypi.python.org/pypi/micc\n\n.. image:: https://img.shields.io/travis/etijskens/micc.svg\n        :target: https://travis-ci.org/etijskens/micc\n\n.. image:: https://readthedocs.org/projects/micc/badge/?version=latest\n        :target: https://micc.readthedocs.io/en/latest/?badge=latest\n        :alt: Documentation Status\n\n\n*Micc* is a Python project manager: it can\n\n* create new projects\n* add python modules and applications\n* add and build binary extension modules written in C++ and Fortran.\n* build documentation (using `sphinx <http://www.sphinx-doc.org/>`_)\n* do version management and control (using `git <https://www.git-scm.com/>`_)\n* test your code (using `pytest <https://www.git-scm.com/>`_)\n\n\nTODO \n\n* facilitates publishing, e.g. `readthedocs <https://readthedocs.org>`_, `PyPI <https://pypi.org>`_  \n* facilitates CI\n\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n',
    'author': 'Engelbert Tijskens',
    'author_email': 'engelbert.tijskens@uantwerpen.be',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/etijskens/et-micc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
